/**
 * A module defining `ModbusWritePointsConfig`.
 *
 * @module nmodule/honIrmControl/rc/ModbusWritePointsConfig
 */
define(['baja!',
        'bajaux/Widget',
        'css!nmodule/honIrmControl/rc/css/irmnx-styles',
        'css!nmodule/honIrmControl/rc/css/comp-table',
        'nmodule/honIrmControl/rc/ModbusUtil',
        'lex!honIrmControl',
        'dialogs',
        'jquery'], function ( 
        baja,
        Widget,
        css,
        tablecss,
        ModbusUtil,
        lexicons,
        dialogs,
        $ ) {
	
	
	  'use strict';	
	  var honIrmControlLex = lexicons[0];
	
	var ModbusWritePointsConfig = function ModbusWritePointsConfig() {
	      Widget.apply(this, arguments);
	  };
	  
	  ModbusWritePointsConfig.prototype = Object.create(Widget.prototype);
	  ModbusWritePointsConfig.prototype.constructor = ModbusWritePointsConfig;  
	  
	  var modbusUtil = new ModbusUtil();
	  
	  
	  ModbusWritePointsConfig.prototype.getWritePointTableId = function(dom) {		
			return dom.find('#writePointTableBodyId');
		}
	  
	    //read pts filter
      ModbusWritePointsConfig.writePtAddressFilterDsc = "writePtAddressFilterDsc";
	  ModbusWritePointsConfig.writePtAddressFilterAsc = "writePtAddressFilterAsc";
	  ModbusWritePointsConfig.writePtNameFilterDsc = "writePtNameFilterDsc";
	  ModbusWritePointsConfig.writePtNameFilterAsc = "writePtNameFilterAsc";
	  ModbusWritePointsConfig.activeFilterWritePts = ModbusWritePointsConfig.writePtAddressFilterAsc;
	  ModbusWritePointsConfig.writePtTypeFilterDsc = "writePtTypeFilterDsc";
	  ModbusWritePointsConfig.writePtTypeFilterAsc = "writePtTypeFilterAsc";
	 
	
	  ModbusWritePointsConfig.selectAllWritePtIdChkBox = '#selecetAllWritePtIdChkBox';
	  ModbusWritePointsConfig.checkBoxWritePoint = '#checkBoxWritePoint';

	  function sortWriteTableByFilter(ptsArray,filter){
		  
		  if(filter === ModbusWritePointsConfig.writePtAddressFilterAsc){
			  ptsArray.sort(function(writePoint, writePointNext){
	    	    return writePoint.writePtAddress-writePointNext.writePtAddress;
	    	});
		  }else if(filter === ModbusWritePointsConfig.writePtAddressFilterDsc){
			  ptsArray.sort(function(writePoint, writePointNext){
		    	    return writePointNext.writePtAddress-writePoint.writePtAddress;
		    });  
		  }else if(filter === ModbusWritePointsConfig.writePtNameFilterDsc){
			  ptsArray.sort(function(writePoint, writePointNext){
		    	    return writePointNext.writePtName.localeCompare(writePoint.writePtName);
		   });
		  }else if(filter === ModbusWritePointsConfig.writePtNameFilterAsc){
			  ptsArray.sort(function(writePoint, writePointNext){
		    	    return writePoint.writePtName.localeCompare(writePointNext.writePtName);
		    });
		  }else if(filter === ModbusWritePointsConfig.writePtTypeFilterDsc){
			  ptsArray.sort(function(writePoint, writePointNext){
		    	    return writePointNext.writePtTypeValue.localeCompare(writePoint.writePtTypeValue);
		   });
		  }else if(filter === ModbusWritePointsConfig.writePtTypeFilterAsc){
			  ptsArray.sort(function(writePoint, writePointNext){
		    	    return writePoint.writePtTypeValue.localeCompare(writePointNext.writePtTypeValue);
		    });
		  }
	      
	      return ptsArray;
	  }
	  
	  ModbusWritePointsConfig.prototype.sortWriteTableOnSelectedFilter = function(widget,filter){
	      var dom = widget.jq();
	      
	      var writePtsArray = sortWriteTableByFilter(Object.values(widget.writePointDataList),filter);
		  	      
		  var writePointTableBody = this.getWritePointTableId(dom);
	      var writePointRows = writePointTableBody.find(".md-table-row");
		  var totalNoOfWritePts = writePointRows.length;
		  var objWritePointValues = Object.assign({},writePointRows);
		  
		  
	      for (var i = 0; i <totalNoOfWritePts; i++) {   
	    	  var writePointRowIdHash = writePtsArray[i].rowIdJq;
	    	  var writePointRowId = writePointRowIdHash.substring(1,writePointRowIdHash.length);
	    	  objWritePointValues[i].setAttribute("id",writePointRowId);	  	   
		  	 var writePointcells = objWritePointValues[i].getElementsByClassName("modbus-comp-table-cell");
			  
			  for (var l = 0; l <writePointcells.length; l++) { // 0 1 2 3
					 if( l==0 ){
						 writePointcells[l].innerText = writePtsArray[i].writePtName;
						 writePointcells[l].setAttribute("data",writePtsArray[i].writePtName);
						 writePointcells[l].setAttribute("title",writePtsArray[i].writePtName);
					 }
					 else if(l==1){
						 writePointcells[l].innerText = writePtsArray[i].writePtTypeValue;
						 writePointcells[l].setAttribute("data",writePtsArray[i].writePtTypeOrdinal);
						 writePointcells[l].setAttribute("title",writePtsArray[i].writePtTypeValue);
					 }			 
					 else if(l==2){
						 
						 writePointcells[l].setAttribute("data", writePtsArray[i].writePtAddress);					 
						 if(writePtsArray[i].writePtAddress.toString().toUpperCase().includes("0X")){
							 writePointcells[l].innerText = modbusUtil.showAddressInDecAndHexFromHex( writePtsArray[i].writePtAddress);
						 }else{
							 writePointcells[l].innerText = modbusUtil.showAddressInDecAndHex( writePtsArray[i].writePtAddress);
						 }
						 
					 }
					 else if(l==3){			 
						 writePointcells[l].innerText = writePtsArray[i].writePtSignificantChange;
						 writePointcells[l].setAttribute("data",writePtsArray[i].writePtSignificantChange); 
					 } 
					 else if(l==4){
						 writePointcells[l].innerText = writePtsArray[i].writePtDataFormatValue;
						 writePointcells[l].setAttribute("data",writePtsArray[i].writePtDataFormatOrdinal); 
					 }
					 else if(l==5){		 
						  var countStr= writePointRowId.substring(ModbusUtil.eighteenNumber),
						  count = Number(countStr);
						  var writePointTableCheckRowId = "checkBoxWritePoint"+count;						 
						  writePointcells[l].getElementsByTagName("input")[0].setAttribute("id",writePointTableCheckRowId);
					 } 
				  }  
		  
		  	  }				  

	  }	  
	  
	  ModbusWritePointsConfig.prototype.checkAllCheckBoxSelectedFromWriteTable = function(widget){
	  //If select all button is checked
	      var dom = widget.jq()
		  var writePointTableBody = this.getWritePointTableId(dom);
	      var writePointRows = writePointTableBody.find(".md-table-row");
		  var totalNoOfWritePts = writePointRows.length;
		  for (var i = 0; i <totalNoOfWritePts; i++) {
			  var writePointRowId = writePointRows[i].getAttribute("id");
			  var count= writePointRowId.substring(ModbusUtil.eighteenNumber),
			  count = Number(count);
			  var writePointTableCheckRowId = ModbusWritePointsConfig.checkBoxWritePoint+count;
			  if(dom.find(writePointTableCheckRowId).prop("checked") == false){
				  return false;
			  }	  
		  }		  
		  return true;
	  }
	  
	  ModbusWritePointsConfig.prototype.handleSelectAllCheckBoxStatusFromWritePointsTable= function(writePtRowChkBox,widget){
		  if($(writePtRowChkBox).prop("checked") == false){				  
			  $(ModbusWritePointsConfig.selectAllWritePtIdChkBox).prop("checked",false);			  
		     }else{
		    	 if(this.checkAllCheckBoxSelectedFromWriteTable(widget) == true)
	    		 {
	    		  $(ModbusWritePointsConfig.selectAllWritePtIdChkBox).prop("checked",true);		
	    		 }
	     }			  
		  }
	  
	  ModbusWritePointsConfig.prototype.selectAllCheckBoxFromWritePointsTable = function(widget){
	  //If select all button is checked
	      var dom = widget.jq()
		  var writePointTableBody = this.getWritePointTableId(dom);
	      var writePointRows = writePointTableBody.find(".md-table-row");
		  var totalNoOfWritePts = writePointRows.length;
		  var objWritePointValues = Object.assign({},writePointRows);
		  for (var i = 0; i <totalNoOfWritePts; i++) {
			  var writePointRowId = objWritePointValues[i].getAttribute("id");
			  var count= writePointRowId.substring(ModbusUtil.eighteenNumber),
			  count = Number(count);
			  var writePointTableCheckRowId = ModbusWritePointsConfig.checkBoxWritePoint+count;
			  if(dom.find(ModbusWritePointsConfig.selectAllWritePtIdChkBox).prop("checked") == true){
				  dom.find(writePointTableCheckRowId).prop("checked",true);
			  }else{
				  dom.find(writePointTableCheckRowId).prop("checked",false);
			  }		  
		  }			  
		//selected check box row will be deleted  
	  }
	  
	  
	  ModbusWritePointsConfig.prototype.validateWritePtAddressAndType = function(modBusPtsReadList,modBusPt,modBusPtsWriteList){
		  for(var i in modBusPtsWriteList){
			  
			  if(modBusPtsWriteList[i].rowIdJq == modBusPt.rowIdJq){
				  continue;
			  }
			  
			  if(modBusPtsWriteList[i].writePtAddress == modBusPt.writePtAddress){

				  if(modBusPtsWriteList[i].writePtTypeOrdinal == modBusPt.writePtTypeOrdinal){
					  var isErrorShown = modbusUtil.showErrorDiaogOnAddEditPointOnAdress(false,modBusPtsWriteList[i].writePtAddress,modBusPtsWriteList[i].writePtName);
					  return isErrorShown;
				  }
			  }
			  if(modBusPtsWriteList[i].writePtName === modBusPt.writePtName){
				  var isSameNameErrorShown = modbusUtil.showErrorDiaogOnAddEditPointSameName(false,modBusPt.writePtName);
				  return isSameNameErrorShown;
			  }
		  } 
		  
		  for(var j in modBusPtsReadList){
			  if(modBusPtsReadList[j].readPtName === modBusPt.writePtName){
				  var isSameNameErrorShownWritePt = modbusUtil.showErrorDiaogOnAddEditPointSameName(false,modBusPt.writePtName);
				  return isSameNameErrorShownWritePt;
			  } 
		  }
		  return false;	  
		  
	  }
	  
	  ModbusWritePointsConfig.prototype.showErrorDiaogOnAddEditWritePoint = function(significantChange,writePointAddress,writePointName){
		  
		  var significantChangeEmpty =  modbusUtil.checkStirngAsNullOrEmpty(significantChange);
		  var writePointAddressEmpty =  modbusUtil.checkStirngAsNullOrEmpty(writePointAddress)
		  var writePointNameEmpty =  modbusUtil.checkStirngAsNullOrEmpty(writePointName)
		  if(significantChangeEmpty == true || writePointAddressEmpty == true || writePointNameEmpty == true){
			  dialogs.showOk({
				  title: honIrmControlLex.get('honIrmControlLex.writepoint.add.edit.error.title'),
				  content: honIrmControlLex.get('honIrmControlLex.writepoint.add.edit.error'),
				  private: false
			   })
			 .ok(function (dlg) {
				    return true;
				  });
			 return true;
		  } 
		  
		  //writePt address hex or decimal and also the validation range will very
		  //check for read point address nan value
		  if(isNaN(writePointAddress)){
				 dialogs.showOk({
					  title: honIrmControlLex.get('honIrmControlLex.writepoint.add.edit.error.title'),
					  content: honIrmControlLex.get('honIrmControlLex.writepoint.add.edit.writePointAddressVal.nan.error'),
					  private: false
				     })
					  .ok(function () {
					    return true;
					  });
					
				 return true;
		  }
		  //check for limit value
		  if(writePointAddress.toString().toUpperCase().includes("0X")){
			  if(writePointAddress > 0xFFFF || writePointAddress < 0) {
					 dialogs.showOk({
						  title: honIrmControlLex.get('honIrmControlLex.writepoint.add.edit.error.title'),
						  content: honIrmControlLex.get('honIrmControlLex.writepoint.add.edit.writePointAddressVal.hex.limit.error'),
						  private: false
					     })
						  .ok(function () {
						    return true;
						  });
						
					 return true;
				  
			  }
		  }else{
			  if(writePointAddress > 65535 || writePointAddress < 0 ){
					 dialogs.showOk({
						  title: honIrmControlLex.get('honIrmControlLex.writepoint.add.edit.error.title'),
						  content: honIrmControlLex.get('honIrmControlLex.writepoint.add.edit.writePointAddressVal.dec.limit.error'),
						  private: false
					     })
						  .ok(function () {
						    return true;
						  });
						
					 return true;
				  
			  }
		  }
		  
	  }
	  
	  ModbusWritePointsConfig.prototype.getModbusDeleteWritePointTableData = function(widget) {

		  var deleteWritePointList = widget.deleteWritePointDataList;
		  var modbusDeviceOutputContentsObj = baja.$('baja:Component');
		  for (var key in deleteWritePointList) {
			  var modbusDeviceDeleteWritePtArgsObj = baja.$('honIrmControl:ModbusWritePointDisplayArgs');
			  //while adding add the slot name with escaping the special character but while showing to user unescape and show to user
			  modbusDeviceDeleteWritePtArgsObj.setWritePointName( baja.SlotPath.escape(deleteWritePointList[key].writePtName));
			  modbusDeviceDeleteWritePtArgsObj.setOldFb(deleteWritePointList[key].oldFb);
			  modbusDeviceDeleteWritePtArgsObj.setOldPointName(deleteWritePointList[key].oldPointName);
			  modbusDeviceDeleteWritePtArgsObj.setExistingSlotPath(deleteWritePointList[key].existingSlotPath);		  

			  //while adding add the slot name with escaping the special character but while showing to user unescape and show to user
			  modbusDeviceOutputContentsObj.add({
				    slot: modbusDeviceDeleteWritePtArgsObj.getWritePointName(),
				    value: modbusDeviceDeleteWritePtArgsObj
				  })
				  .then(function (prop) {
					})
				  .catch(function (err) {
					   baja.error(err);				      
				  });
			} 
		  
		  return modbusDeviceOutputContentsObj;
	  }
	  
	  
	  
	  ModbusWritePointsConfig.prototype.getModbusWritePointTableData = function(dom, widget) {

		  var writePointList = widget.writePointDataList;
		  var modbusDeviceOutputContentsObj = baja.$('baja:Component');
		  var ptNameArr = [];
		  for (var key in writePointList) {
			  var modbusDeviceWritePtArgsObj = baja.$('honIrmControl:ModbusWritePointDisplayArgs');
			  //while adding add the slot name with escaping the special character but while showing to user unescape and show to user
			  modbusDeviceWritePtArgsObj.setWritePointName( baja.SlotPath.escape(writePointList[key].writePtName));
			  ptNameArr.push(writePointList[key].writePtName);
			  var writePointTypeEnum = baja.DynamicEnum.make(parseInt(writePointList[key].writePtTypeOrdinal));
			  modbusDeviceWritePtArgsObj.setWritePointType(writePointTypeEnum);

			  if(writePointList[key].writePtAddress.toString().toUpperCase().includes("0X")){
				  modbusDeviceWritePtArgsObj.setWritePointAddress(parseInt(writePointList[key].writePtAddress.toString(),16));
			  }else{
				  modbusDeviceWritePtArgsObj.setWritePointAddress(writePointList[key].writePtAddress);  
			  }
			  
			  modbusDeviceWritePtArgsObj.setSignificantChange(writePointList[key].writePtSignificantChange);
			  var dataFormatTypeEnum = baja.DynamicEnum.make(parseInt(writePointList[key].writePtDataFormatOrdinal));
			  modbusDeviceWritePtArgsObj.setDataFormat(dataFormatTypeEnum);
			  modbusDeviceWritePtArgsObj.setOldFb(writePointList[key].oldFb);
			  modbusDeviceWritePtArgsObj.setOldPointName(writePointList[key].oldPointName);
			  modbusDeviceWritePtArgsObj.setExistingSlotPath(writePointList[key].existingSlotPath);		  

			  //while adding add the slot name with escaping the special character but while showing to user unescape and show to user
			  modbusDeviceOutputContentsObj.add({
				    slot: modbusDeviceWritePtArgsObj.getWritePointName(),
				    value: modbusDeviceWritePtArgsObj
				  })
				  .then(function (prop) {
					})
				  .catch(function (err) {
					   baja.error(err);				      
				  });
			} 
			for (var i = 0; i < ptNameArr.length; i++) { 
		   		   for(var j = i+1; j < ptNameArr.length; j++){
		   			   if(ptNameArr[i] === ptNameArr[j]){
		   				var isErrorShown = modbusUtil.showErrorDiaogOnAddEditPointSameName(true,ptNameArr[i]);
		   				if(isErrorShown == true){
		   					return true;
		   				}
		   			  }
		   		   }
		   		}
		  
		  return modbusDeviceOutputContentsObj;
	  }
	
	
	  return ModbusWritePointsConfig;
	
});
