/*
Copyright (c) 2021 Honeywell International, Inc. All rights reserved.
This file contains trade secrets of Honeywell International, Inc. No part
may be reproduced or transmitted in any form by any means or for any
purpose without the express written permission of Honeywell.
*/
/**
  * Builds a javascript widget that builds property sheet for Auto Synch inteval slot of Floating Block
  * @E579516
  */
define(['baja!',
  'nmodule/webEditors/rc/fe/baja/BaseEditor', 
  'baja!honIrmControl:AutoSyncIntervalEnum',
  'bajaux/mixin/subscriberMixIn',
  'jquery',
  'dialogs',
  'Promise',
  'bajaux/util/SaveCommand',
  'hbs!nmodule/honIrmControl/rc/templates/autoSynchInterval',
  'lex!honIrmControl'],
  function (      //NOSONAR
    baja,
    BaseEditor,
    AutoSyncIntervalEnum,
    subscriberMixin,
    $,
    dialogs,
    Promise,     //NOSONAR
    SaveCommand,
    template,
    lex) {
	
	'use strict';
	
	var honIrmControlLex = lex[0];
	
	
	function getAutoSynchIntevalSelect(dom) {									
		return dom.find('#autoSynchIntevalSelectId');
	}
    
	function getAutoSynchIntevalTxt(dom) {									
		return  dom.find('#autoSynchIntevalTxtId');
	}
	
	function getAutoSynchIntevalTxtLbl(dom) {									
		return dom.find('#autoSynchIntevalTxtLblId');
	}
	
	function getAutoSynchIntevalTxtUnitLbl(dom) {									
		return dom.find('#autoSynchIntevalTxtUnitLblId');
	}
	
    /**
    * An editor which helps in editing value
    */
    var AutoSynchIntervalWidget = function () {   //NOSONAR
      BaseEditor.apply(this, arguments);
    	//BaseEditor.make(this,arguments);
      subscriberMixin(this);
      this.getCommandGroup().add(new SaveCommand());
      this.autoSyncIntervalOptionsObjs = baja.$('honIrmControl:AutoSyncIntervalEnum');
      this.NoTimeBasedRepeatedSynchronization = this.autoSyncIntervalOptionsObjs.getEnum().getRange().getOrdinals()[0];
      this.ToClosedOrOpenDependingOnSynchOperation = this.autoSyncIntervalOptionsObjs.getEnum().getRange().getOrdinals()[1];
      this.ToClosedIndependentOfSynchOperation = this.autoSyncIntervalOptionsObjs.getEnum().getRange().getOrdinals()[2];
      this.ToOpenIndependentOfSynchOperation = this.autoSyncIntervalOptionsObjs.getEnum().getRange().getOrdinals()[3];
      this.ToClosedAfter24hRandomly = this.autoSyncIntervalOptionsObjs.getEnum().getRange().getOrdinals()[4];
      this.ToOpenAfter24hRandomly = this.autoSyncIntervalOptionsObjs.getEnum().getRange().getOrdinals()[5];
      this.autoSyncIntervalMaxVal =255; 
      this.autoSyncIntervalMinVal =1; 
    };

    //extend and set up prototype chain
    AutoSynchIntervalWidget.prototype = Object.create(BaseEditor.prototype);
    AutoSynchIntervalWidget.prototype.constructor = AutoSynchIntervalWidget;

    /**
    * Do initial setup of the DOM for the widget. This will set up the DOM's
    * structure and create a space where the editors would go.
    *
    * @param {jQuery} element the DOM element into which to load this widget
    */
    AutoSynchIntervalWidget.prototype.doInitialize = function (dom) {
      BaseEditor.prototype.doInitialize.call(this, dom);
      var that = this;     
      dom.html(template({
    	  
      }));  
       
      dom.find('select').on('change', function () {
          that.setModified(true);
        });  
      dom.find('input').on('change', function () {
          that.setModified(true);
        });
    };
    
    /**
    * Load the component
    */
    AutoSynchIntervalWidget.prototype.doLoad = function (comp) {
  	 var widget = this;
  	 var dom = widget.jq();
 	 loadAutoSynchIntevalContent(comp, widget);
	 dom.on("change", "#autoSynchIntevalSelectId",widget,function(event){
		     event.preventDefault();
		     changeVisibilityOfAutoSynchIntevalTextField(widget);
		});
	 dom.on("change", "#autoSynchIntevalTxtId",widget,function(event){
	     event.preventDefault();
	     var isErrorDialogShown = showErrorOnAutoSynchIntevalValidation(widget);
	     if(true == isErrorDialogShown){
	    	 getAutoSynchIntevalTxt(dom).val(Number(widget.autoSyncIntervalMaxVal));
	     }
	     
	});
    };
    
    
    
   function showErrorOnAutoSynchIntevalValidation(widget){  
	var dom = widget.jq();
    var autoSynchIntevalOption = getAutoSynchIntevalSelect(dom).val();
    if(autoSynchIntevalOption == widget.ToClosedOrOpenDependingOnSynchOperation
    		|| autoSynchIntevalOption == widget.ToClosedIndependentOfSynchOperation
    		|| autoSynchIntevalOption == widget.ToOpenIndependentOfSynchOperation){
   	 var autoSynchIntevalOptionValueEntered = getAutoSynchIntevalTxt(dom).val();
		  if(autoSynchIntevalOptionValueEntered > widget.autoSyncIntervalMaxVal || autoSynchIntevalOptionValueEntered < widget.autoSyncIntervalMinVal ){
				 dialogs.showOk({
					  title: honIrmControlLex.get('honIrmControlLex.floating.error.title'),
					  content: honIrmControlLex.get('honIrmControlLex.auto.synch.inteval.limit.error'),
					  private: false
				     })
					  .ok(function () {
					    return true;
					  });
					
				 return true;
			  
		  }
     }
    }
    
    function changeVisibilityOfAutoSynchIntevalTextField(widget){
    	 var dom = widget.jq();
	     var autoSynchIntevalOption = getAutoSynchIntevalSelect(dom).val();
	     if(autoSynchIntevalOption == widget.NoTimeBasedRepeatedSynchronization
	    		 || autoSynchIntevalOption == widget.ToClosedAfter24hRandomly
	    		 || autoSynchIntevalOption == widget.ToOpenAfter24hRandomly ){
	    	 getAutoSynchIntevalTxt(dom).hide();
	    	 getAutoSynchIntevalTxtLbl(dom).hide();
	    	 getAutoSynchIntevalTxtUnitLbl(dom).hide();
	    	 if(autoSynchIntevalOption == widget.NoTimeBasedRepeatedSynchronization ){
	    		 getAutoSynchIntevalTxt(dom).val(Number(widget.NoTimeBasedRepeatedSynchronization)); 
	    	 }else if(autoSynchIntevalOption == widget.ToClosedAfter24hRandomly ){
	    		 getAutoSynchIntevalTxt(dom).val(Number(widget.ToClosedAfter24hRandomly));  
	    	 }else if(autoSynchIntevalOption == widget.ToOpenAfter24hRandomly ){
	    		 getAutoSynchIntevalTxt(dom).val(Number(widget.ToOpenAfter24hRandomly)); 
	    	 }
	    	 
	     }else{
	    	 getAutoSynchIntevalTxt(dom).val(Number(widget.autoSyncIntervalMaxVal));
	    	 getAutoSynchIntevalTxt(dom).show();
	    	 getAutoSynchIntevalTxtLbl(dom).show();
	    	 getAutoSynchIntevalTxtUnitLbl(dom).show();
	     }
    }
    
    function loadAutoSynchIntevalContent(component, widget){
    	var jsonObj = null;
    	widget.getComplex().getParent().rpc("getAutoSynchIntevalValues").then(function(value) {
    		if(value !== undefined){
    			 var dom = widget.jq();
    			jsonObj = JSON.parse(value);
    			 var autoSynchIntervalEnum = jsonObj.autoSynchIntervalEnum,
    			 autoSynchIntervalValue = jsonObj.autoSynchIntervalValue;
    			 var autoSyncIntervalOptionsObjs1 = baja.$('honIrmControl:AutoSyncIntervalEnum');
    			 var autoSyncIntervalOptionOrdinal = autoSyncIntervalOptionsObjs1.getEnum().getRange().getOrdinals();
    				var option = '';
    				$("#autoSynchIntevalSelectId option").remove();
    				for (var i=0;i<autoSyncIntervalOptionOrdinal.length;i++){
    					var autoSyncIntervalOption =  autoSyncIntervalOptionsObjs1.getEnum().getRange().getDisplayTag(autoSyncIntervalOptionOrdinal[i]);
    					option += '<option value="'+ autoSyncIntervalOptionOrdinal[i] + '">' + autoSyncIntervalOption + '</option>';
    				}
    				getAutoSynchIntevalSelect(dom).append(option);
    				getAutoSynchIntevalSelect(dom).val(autoSynchIntervalEnum.toString());
    				getAutoSynchIntevalTxt(dom).val(Number(autoSynchIntervalValue));
    			     if(autoSynchIntervalEnum == widget.NoTimeBasedRepeatedSynchronization
    			    		 || autoSynchIntervalEnum == widget.ToClosedAfter24hRandomly
    			    		 || autoSynchIntervalEnum == widget.ToOpenAfter24hRandomly ){
    			    	 getAutoSynchIntevalTxt(dom).hide();
    			    	 getAutoSynchIntevalTxtLbl(dom).hide();
    			    	 getAutoSynchIntevalTxtUnitLbl(dom).hide();
    			     }else{
    			    	 getAutoSynchIntevalTxt(dom).show();
    			    	 getAutoSynchIntevalTxtLbl(dom).show();
    			    	 getAutoSynchIntevalTxtUnitLbl(dom).show();
    			     }
    		}        	    	
        });
    	
    }
    
    /**
     * Detach the event handlers when destroying the widget.
     */
    AutoSynchIntervalWidget.prototype.doDestroy = function () {
      this.jq().find('input').off();
      this.jq().find('select').off();
    };

    /**
     * Save the user-entered changes to the loaded component.
     *
     * Note that the parameter to this function is the same as that resolved by
     * doRead().
     *
     */
    AutoSynchIntervalWidget.prototype.doSave = function (readValue, params) {
     	 var widget = this;
      	 saveAutoSynchIntervalContent(readValue, widget);
     };
       
     function saveAutoSynchIntervalContent(component, widget){
    	 var dom = widget.jq();
    	 var autoSynchInteval = { autoSynchIntervalEnum: getAutoSynchIntevalSelect(dom).val(),autoSynchIntervalValue: getAutoSynchIntevalTxt(dom).val()};
    	 var autoSynchIntevalJSON = JSON.stringify(autoSynchInteval);
       widget.getComplex().getParent().rpc("setAutoSynchIntevalValues",autoSynchIntevalJSON).then(function(value){     	    	
          });
      }
    return AutoSynchIntervalWidget;
  });
