/*
Copyright (c) 2021 Honeywell International, Inc. All rights reserved.
This file contains trade secrets of Honeywell International, Inc. No part
may be reproduced or transmitted in any form by any means or for any
purpose without the express written permission of Honeywell.
*/
/**
  * Builds a javascript widget that builds property sheet for Power Up Delay slot of Floating Block
  * @E579516
  */
define(['baja!',
  'nmodule/webEditors/rc/fe/baja/BaseEditor',
  'baja!honIrmControl:PowerUPDelayEnum',
  'bajaux/mixin/subscriberMixIn',
  'jquery',
  'dialogs',
  'Promise',
  'bajaux/util/SaveCommand',
  'hbs!nmodule/honIrmControl/rc/templates/powerUpDelay',
  'lex!honIrmControl'],
  function (      //NOSONAR
    baja,
    BaseEditor,
    PowerUPDelayEnum,
    subscriberMixin,
    $,
    dialogs,
    Promise,     //NOSONAR
    SaveCommand,
    template,
    lex) {
	
	'use strict';
	
	var honIrmControlLex = lex[0];
	
	
	function getPowerUpDelaySelect(dom) {									
		return dom.find('#powerUpDelaySelectId');
	}
    
	function getPowerUpDelayTxt(dom) {									
		return  dom.find('#powerUpDelayTxtId');
	}
	
	function getPowerUpDelayTxtLbl(dom) {									
		return dom.find('#powerUpDelayTxtLblId');
	}
	
	
	function getPowerUpDelayTxtUnitLbl(dom) {									
		return dom.find('#powerUpDelayTxtUnitLblId');
	}
	

	

    /**
    * An editor which helps in editing value
    */
    var PowerUpDelayWidget = function () {   //NOSONAR
      BaseEditor.apply(this, arguments);
    	//BaseEditor.make(this,arguments);
      subscriberMixin(this);
      this.getCommandGroup().add(new SaveCommand());
      this.powerUpDelayOptionsObjs = baja.$('honIrmControl:PowerUPDelayEnum');
      this.NoPowerUpDelay = this.powerUpDelayOptionsObjs.getEnum().getRange().getOrdinals()[0];
      this.WaitAccordingToTheSelectedPowerUpDelay = this.powerUpDelayOptionsObjs.getEnum().getRange().getOrdinals()[1];
      this.MoveToClosedAfterPowerUpWithoutWaiting = this.powerUpDelayOptionsObjs.getEnum().getRange().getOrdinals()[2];
      this.MoveToOpenAfterPowerUpWithoutWaiting = this.powerUpDelayOptionsObjs.getEnum().getRange().getOrdinals()[3];
      this.SynchronizeRandomlyToClosedOrOpenWithoutWaiting = this.powerUpDelayOptionsObjs.getEnum().getRange().getOrdinals()[4];
      this.powerUpDelayMaxVal =4095; 
      this.powerUpDelayMinVal =1; 
    };

    //extend and set up prototype chain
    PowerUpDelayWidget.prototype = Object.create(BaseEditor.prototype);
    PowerUpDelayWidget.prototype.constructor = PowerUpDelayWidget;

    /**
    * Do initial setup of the DOM for the widget. This will set up the DOM's
    * structure and create a space where the editors would go.
    *
    * @param {jQuery} element the DOM element into which to load this widget
    */
    PowerUpDelayWidget.prototype.doInitialize = function (dom) {
      BaseEditor.prototype.doInitialize.call(this, dom);
      var that = this;     
      dom.html(template({
    	  
      }));  
       
      dom.find('select').on('change', function () {
          that.setModified(true);
        });  
      dom.find('input').on('change', function () {
          that.setModified(true);
        });
    };
    
    /**
    * Load the component
    */
    PowerUpDelayWidget.prototype.doLoad = function (comp) {
  	 var widget = this;
  	 var dom = widget.jq();
 	 loadPowerUpDelayContent(comp, widget);
	 dom.on("change", "#powerUpDelaySelectId",widget,function(event){
		     event.preventDefault();
		     changeVisibilityOfPowerUpDelayTextField(widget);
		});
	 dom.on("change", "#powerUpDelayTxtId",widget,function(event){
	     event.preventDefault();
	     var isErrorDialogShown = showErrorOnPowerUpDelayValidation(widget);
	     if(true == isErrorDialogShown){
	    	 getPowerUpDelayTxt(dom).val(Number(widget.powerUpDelayMaxVal));
	     }
	     
	});
    };
    
    
    
   function showErrorOnPowerUpDelayValidation(widget){ 
	var dom = widget.jq();
    var powerUpDelayOption = getPowerUpDelaySelect(dom).val();
    if(powerUpDelayOption == widget.WaitAccordingToTheSelectedPowerUpDelay){
   	 var powerUpDelayOptionValueEntered = getPowerUpDelayTxt(dom).val();
		  if(powerUpDelayOptionValueEntered > widget.powerUpDelayMaxVal || powerUpDelayOptionValueEntered < widget.powerUpDelayMinVal ){
				 dialogs.showOk({
					  title: honIrmControlLex.get('honIrmControlLex.floating.error.title'),
					  content: honIrmControlLex.get('honIrmControlLex.power.up.delay.limit.error'),
					  private: false
				     })
					  .ok(function () {
					    return true;
					  });
					
				 return true;
			  
		  }
     }
    }
    
    function changeVisibilityOfPowerUpDelayTextField(widget){ 
    	 var dom = widget.jq();
	     var powerUpDelayOption = getPowerUpDelaySelect(dom).val();
	     if(powerUpDelayOption != widget.WaitAccordingToTheSelectedPowerUpDelay){
	    	 getPowerUpDelayTxt(dom).hide();
	    	 getPowerUpDelayTxtLbl(dom).hide();
	    	 getPowerUpDelayTxtUnitLbl(dom).hide();
	    	 if(powerUpDelayOption == widget.NoPowerUpDelay ){
	    	 getPowerUpDelayTxt(dom).val(Number(widget.NoPowerUpDelay)); 
	    	 }else if(powerUpDelayOption == widget.MoveToClosedAfterPowerUpWithoutWaiting ){
	    		getPowerUpDelayTxt(dom).val(Number(widget.MoveToClosedAfterPowerUpWithoutWaiting));  
	    	 }else if(powerUpDelayOption == widget.MoveToOpenAfterPowerUpWithoutWaiting ){
	    		 getPowerUpDelayTxt(dom).val(Number(widget.MoveToOpenAfterPowerUpWithoutWaiting)); 
	    	 }else{
	    		 getPowerUpDelayTxt(dom).val(Number(widget.SynchronizeRandomlyToClosedOrOpenWithoutWaiting)); 
	    	 }
	    	 
	     }else{
	    	 getPowerUpDelayTxt(dom).val(Number(widget.powerUpDelayMaxVal));
	    	 getPowerUpDelayTxt(dom).show();
	    	 getPowerUpDelayTxtLbl(dom).show();
	    	 getPowerUpDelayTxtUnitLbl(dom).show();
	     }
    }
    
    function loadPowerUpDelayContent(component, widget){
    	widget.getComplex().getParent().rpc("getPowerUpDelayValues").then(function(value) {
    		if(value !== undefined){
    			 var dom = widget.jq();
    			var jsonObj = JSON.parse(value);
    			 var powerUpDelayEnum = jsonObj.powerUpDelayEnum,
    			 powerUpDelayValue = jsonObj.powerUpDelayValue

    			 var powerUpDelayOptionsObj1s = baja.$('honIrmControl:PowerUPDelayEnum');
    			 var powerUpDelayOptionOrdinal = powerUpDelayOptionsObj1s.getEnum().getRange().getOrdinals();
    				var option = '';
    				$("#powerUpDelaySelectId option").remove();
    				for (var i=0;i<powerUpDelayOptionOrdinal.length;i++){
    					var powerUpDelayOption =  powerUpDelayOptionsObj1s.getEnum().getRange().getDisplayTag(powerUpDelayOptionOrdinal[i]);
    					option += '<option value="'+ powerUpDelayOptionOrdinal[i] + '">' + powerUpDelayOption + '</option>';
    				}
    				getPowerUpDelaySelect(dom).append(option);
    				getPowerUpDelaySelect(dom).val(powerUpDelayEnum.toString());
    				getPowerUpDelayTxt(dom).val(Number(powerUpDelayValue));
    			     if(powerUpDelayEnum != widget.WaitAccordingToTheSelectedPowerUpDelay){
    			    	 getPowerUpDelayTxt(dom).hide();
    			    	 getPowerUpDelayTxtLbl(dom).hide();
    			    	 getPowerUpDelayTxtUnitLbl(dom).hide();
    			     }else{
    			    	 getPowerUpDelayTxt(dom).show();
    			    	 getPowerUpDelayTxtLbl(dom).show();
    			    	 getPowerUpDelayTxtUnitLbl(dom).show();
    			     }
    		}        	    	
        });
    }
    
    /**
     * Detach the event handlers when destroying the widget.
     */
    PowerUpDelayWidget.prototype.doDestroy = function () {
      this.jq().find('input').off();
      this.jq().find('select').off();
    };

    /**
     * Save the user-entered changes to the loaded component.
     *
     * Note that the parameter to this function is the same as that resolved by
     * doRead().
     *
     */
    PowerUpDelayWidget.prototype.doSave = function (readValue, params) {
     	 var widget = this;
      	 savePowerUpDelayContent(readValue, widget);
     };
       
     function savePowerUpDelayContent(component, widget){
    	 var dom = widget.jq();
    	 var powerUpDelay = { powerUpDelayValue: getPowerUpDelayTxt(dom).val()};
    	 var powerUpDelayJSON = JSON.stringify(powerUpDelay);
       widget.getComplex().getParent().rpc("setPowerUpDelayValues",powerUpDelayJSON).then(function(value){     	    	
          });
      }
    return PowerUpDelayWidget;
  });
