/*
Copyright (c) 2021 Honeywell International, Inc. All rights reserved.
This file contains trade secrets of Honeywell International, Inc. No part
may be reproduced or transmitted in any form by any means or for any
purpose without the express written permission of Honeywell.
*/

/**
  * Builds a javascript widget that builds Terminal assignment view
  * @H356559
  */

define(['baja!',
	'nmodule/webEditors/rc/fe/baja/BaseEditor',
	'bajaux/mixin/subscriberMixIn',
	'jquery',
	'bajaux/util/SaveCommand',
	'Handlebars',
	'hbs!nmodule/honIrmConfig/rc/templates/beatsVAV',
	'hbs!nmodule/honIrmConfig/rc/templates/beatsLarge164424VFCU',
	'hbs!nmodule/honIrmConfig/rc/templates/beatsLarge168424VFCU',
	'hbs!nmodule/honIrmConfig/rc/templates/beatsSmall084424VFCU',
	'hbs!nmodule/honIrmConfig/rc/templates/compactVAV',
	'hbs!nmodule/honIrmConfig/rc/templates/irmLargeFCU',
	'hbs!nmodule/honIrmConfig/rc/templates/irmRS4NRS5NFCU',
	'hbs!nmodule/honIrmConfig/rc/templates/irmRS3NFCU',
	'hbs!nmodule/honIrmConfig/rc/templates/beatsLarge1644230VFCU',
	'hbs!nmodule/honIrmConfig/rc/templates/beatsSmall0844230VFCU',
	'hbs!nmodule/honIrmConfig/rc/templates/terminalAssignmentView',
	'dialogs',
	'lex!honIrmConfig',
	'Promise',
	'css!nmodule/honIrmConfig/rc/css/terminalassignmentview/beatsVAV',
	'css!nmodule/honIrmConfig/rc/css/terminalassignmentview/beatsLarge164424VFCU',
	'css!nmodule/honIrmConfig/rc/css/terminalassignmentview/beatsLarge168424VFCU',
	'css!nmodule/honIrmConfig/rc/css/terminalassignmentview/beatsSmall084424VFCU',
	'css!nmodule/honIrmConfig/rc/css/terminalassignmentview/compactVAV',
	'css!nmodule/honIrmConfig/rc/css/terminalassignmentview/irmLargeFCU',
	'css!nmodule/honIrmConfig/rc/css/terminalassignmentview/irmRS4NRS5NFCU',
	'css!nmodule/honIrmConfig/rc/css/terminalassignmentview/irmRS3NFCU',
	'css!nmodule/honIrmConfig/rc/css/terminalassignmentview/beatsLarge1644230VFCU',
	'css!nmodule/honIrmConfig/rc/css/terminalassignmentview/beatsSmall0844230VFCU',
	'css!nmodule/honIrmConfig/rc/css/terminalassignmentview/terminalAssignmentView'],
	function(      //NOSONAR
		baja,
		BaseEditor,
		subscriberMixin,
		$,
		SaveCommand,
		Handlebars,
		beatsVAV,
		beatsLarge164424VFCU,
	    beatsLarge168424VFCU,
		beatsSmall084424VFCU,
		compactVAV,
		irmLargeFCU,
		irmRS4NRS5NFCU,
		irmRS3NFCU,
		beatsLarge1644230VFCU,
		beatsSmall0844230VFCU,
		terminalAssignmentView,
		dialogs,
		lexicons) {

		'use strict';

		var honIrmConfigLex = lexicons[0];
		var terminals = [];
		var changedItems = baja.$('baja:Component');
		var unassigned = "Unassigned";
		var underDevelopment = 0;
		var jobInProgress = 1;
		var migrationPending = 2;
		var invalidModel = 3;
		var invalidBrand = 4;
		var notSupported = 5;
		var beatsVAVTemplate = "beatsVAV";
		var beatsLarge164424VFCUTemplate = "beatsLarge164424VFCU";
		var beatsSmall084424VFCUTemplate = "beatsSmall084424VFCU";
		var beatsLarge168424VFCUTemplate = "beatsLarge168424VFCU";
		var compactVAVTemplate = "compactVAV";
		var irmLargeFCUTemplate = "irmLargeFCU";
		var irmRS4NRS5NFCUTemplate = "irmRS4NRS5NFCU";
		var irmRS3NFCUTemplate = "irmRS3NFCU";
		var beatsLarge1644230VFCUTemplate = "beatsLarge1644230VFCU";
		var beatsSmall0844230VFCUTemplate = "beatsSmall0844230VFCU";
		var zoomClicks = 0;
		var minZoomClicks = -3;
		var maxZoomClicks = 10;
		var defaultZoomLevel = 100;
		var dropDownReferencePxSize;
		var sub = new baja.Subscriber();
		
		/**
		* An editor which helps in editing value
		*/
		var TerminalAssignment = function() {   //NOSONAR
			BaseEditor.apply(this, arguments);
			subscriberMixin(this);
			this.getCommandGroup().add(new SaveCommand());

			Handlebars.registerHelper("showTerminalAssignment", function(type) {
				console.log("type to show " + type.data.root.type);
				if (type.data.root.type === beatsVAVTemplate) {
					return new Handlebars.SafeString(beatsVAV({}));
				} else if (type.data.root.type === beatsLarge164424VFCUTemplate) {
					return new Handlebars.SafeString(beatsLarge164424VFCU({}));
				} else if (type.data.root.type === beatsSmall084424VFCUTemplate) {
					return new Handlebars.SafeString(beatsSmall084424VFCU({}));
				} else if (type.data.root.type === compactVAVTemplate) {
					return new Handlebars.SafeString(compactVAV({}));
				} else if (type.data.root.type === irmLargeFCUTemplate) {
					return new Handlebars.SafeString(irmLargeFCU({}));
				} else if (type.data.root.type === irmRS4NRS5NFCUTemplate) {
					return new Handlebars.SafeString(irmRS4NRS5NFCU({}));
				} else if (type.data.root.type === irmRS3NFCUTemplate) {
					return new Handlebars.SafeString(irmRS3NFCU({}));
				} else if (type.data.root.type === beatsLarge1644230VFCUTemplate) {
					return new Handlebars.SafeString(beatsLarge1644230VFCU({}));
				} else if (type.data.root.type === beatsSmall0844230VFCUTemplate) {
					return new Handlebars.SafeString(beatsSmall0844230VFCU({}));
				} else if (type.data.root.type === beatsLarge168424VFCUTemplate) {
				    return new Handlebars.SafeString(beatsLarge168424VFCU({}));
				}
			});
		};

		//extend and set up prototype chain
		TerminalAssignment.prototype = Object.create(BaseEditor.prototype);
		TerminalAssignment.prototype.constructor = TerminalAssignment;

		/**
		* Load the component.
		* Update the terminal Assignement view with the correct HBS file based on response from Server side.
		* Fetch IO Terminal Details from ServerSide based on model and update the details to the dropdown.
		* Subscribe all the Terminals for changed callback.
		*/
		TerminalAssignment.prototype.doLoad = function(comp) {
			var widget = this;
			var dom = widget.jq();
			BaseEditor.prototype.doInitialize.call(this, dom);
			zoomClicks = 0;
			var clickEvent = ['#zoomIn','#zoomOut','#defaultZoom'];
			dom.on("click", clickEvent, widget, function(event) {
				event.preventDefault();
				if (this.id == "zoomIn" && zoomClicks < maxZoomClicks) {
					zoomClicks++;
				} else if (this.id == "zoomOut" && zoomClicks > minZoomClicks) {
					zoomClicks--;
				} else if (this.id == "defaultZoom") {
					zoomClicks = 0;
				}
				setStagedZoom();
			});
			dom.on("keypress", '#customZoom', widget, function(event) {
				if (event.keyCode == 13) {
					event.preventDefault();
					var customZoom = document.getElementById('customZoom');
					setUserZoom(customZoom);
				}
			});
			var result = updateTerminalAssignmentViewAndGetDeviceDetails(comp, dom);
			if (result.$val !== underDevelopment && result.$val !== jobInProgress && result.$val !== migrationPending && result.$val !== invalidBrand && result.$val !== invalidModel && result.$val !== notSupported) {
				getIOTerminalDetails(comp).then(function(ioTerminalPropResult) {
					(loadIOTerminalDropdown(ioTerminalPropResult, dom)).then(subscribeToChangedCallBack(dom, widget)).catch(function(err) {
						return Promise.reject(err);
					});
				}).catch(function(err) {
					return Promise.reject(err);
				});
			}
			sub.attach('changed', function(prop) {
		 		if (prop.getName() === 'isTerminalSaveInProgress') { 
		 		enableOrDisableTheDropDown(prop.$display); 
		 		}
	 		});
	 			 var ord = 'station:|' + comp.getSlotPath();
	 			 baja.Ord.make(ord).get({ subscriber: sub }).then(enableOrDisableTheDropDown);
		};
		
		function enableOrDisableTheDropDown(enableOrDisable){
			for (var index in terminals){
				if (index == "contains") {
						return false;
				}
				var terminal = document.getElementById(terminals[index]);
				if(enableOrDisable === "true"){
					terminal.disabled=true;
				}else{
					terminal.disabled=false;
				}
			}
		}

		function setStagedZoom() {
			var zoomLevel = (defaultZoomLevel + (zoomClicks * 10));
			setZoomLevel(zoomLevel);
			var customZoom = document.getElementById('customZoom');
			customZoom.value = (defaultZoomLevel + (zoomClicks * 10));
		}
		
		function setZoomLevel(zoomLevel) {
			var terminalAssignmentContentView = document.getElementById('mainCont');
			terminalAssignmentContentView.style.width = zoomLevel + "%";
			terminalAssignmentContentView.style.position = "relative";
			terminalAssignmentContentView.style.right = (zoomClicks * 4) + "%";
			for (var index in terminals) {
				if (index == "contains") {
					return false;
				}
				var terminal = document.getElementById(terminals[index]);
				terminal.style.fontSize = (dropDownReferencePxSize + (zoomClicks * 0.8)) + "px";
				terminal.style.verticalAlign = "-" + (zoomClicks * 5) + "%";
			}
		}
		
		function setUserZoom(customZoom) {
			var maxZoomLevel = 200;
			var minZoomLevel = 70;
			if (typeof (customZoom.val == 'number')) {
				if (customZoom.value > maxZoomLevel) {
					zoomClicks = maxZoomClicks;
					customZoom.value = maxZoomLevel;
					setZoomLevel(maxZoomLevel);
				} else if (customZoom.value < minZoomLevel) {
					zoomClicks = minZoomClicks;
					customZoom.value = minZoomLevel;
					setZoomLevel(minZoomLevel);
				} else {
					zoomClicks = Math.round((customZoom.value - defaultZoomLevel) / 10);
					setZoomLevel(customZoom.value);
				}
			} else {
				customZoom.value = (zoomClicks * 10) + defaultZoomLevel;
			}
		}
		
		/**
		* Find and @returns the JQuery element based on the DOM & terminalName
		*/
		function getCorrespondingIOTerminal(dom, terminalName) {
			var term = '#' + terminalName;
			return dom.find(term);
		}

		/**
		* Append the terminalName with '#'.
		* @returns
		*/
		function getCorrespondingTerminalId(terminalName) {
			return ('#' + terminalName);
		}

		/**
		* Query the server side for the device model details and update and the corresponding view based on the response.
		* @returns the result from ServerSide once the operation is complete for further processing.
		*/
		function updateTerminalAssignmentViewAndGetDeviceDetails(comp, dom) {
			var terminalServerSideclassName = "honIrmConfig:TerminalAssignmentServerSideCallHandler";
			var modelDetails = "getCurrentDeviceModelDetails";

			return comp.serverSideCall({
				typeSpec: terminalServerSideclassName,
				methodName: modelDetails
			})
				.then(function(propResult) {	
					console.log("propResult: " + propResult.$val);
					var errorText; 
					if (propResult.$val === underDevelopment) {
						errorText = new Handlebars.SafeString(honIrmConfigLex.get('honIrmConfigLex.under.development'));
					} else if (propResult.$val === jobInProgress) {
						errorText = new Handlebars.SafeString(honIrmConfigLex.get('honIrmConfigLex.job.in.progress'));
					} else if (propResult.$val === migrationPending) {
						errorText = new Handlebars.SafeString(honIrmConfigLex.get('honIrmConfigLex.migration.pending'));
					} else if (propResult.$val === invalidBrand) {
						errorText = new Handlebars.SafeString(honIrmConfigLex.get('honIrmConfigLex.invalid.brand'));
					} else if (propResult.$val === invalidModel) {
						errorText = new Handlebars.SafeString(honIrmConfigLex.get('honIrmConfigLex.invalid.model'));
					} else if (propResult.$val === notSupported) {
						errorText = new Handlebars.SafeString(honIrmConfigLex.get('honIrmConfigLex.not.supported'));
					}

					loadTerminalAssignmentView(errorText, dom, propResult)
					return propResult;
				})
				.catch(function(err) {
					return Promise.reject(err);
				});
		}
		
		function loadTerminalAssignmentView(errorText, dom, propResult) {
			var beatsVAVController = 6;
			var beatsLarge164424VFCUController = 7;
			var beatsSmall084424VFCUController = 8;
			var compactVAVController = 9;
			var irmLargeFCUController = 10;
			var irmRS4NRS5NFCUController = 11;
			var irmRS3NFCUController = 12;
			var beatsLarge1644230VFCUController = 13;
			var beatsSmall0844230VFCUController = 14;
			var beatsLarge168424VFCUController = 15;

			
			if (errorText) {
				dom.html(terminalAssignmentView({
					errorText: errorText
				}));
			} else if (propResult.$val === beatsVAVController) {
				dom.html(terminalAssignmentView({ type: beatsVAVTemplate }));
			} else if (propResult.$val === beatsLarge164424VFCUController) {
				dom.html(terminalAssignmentView({ type: beatsLarge164424VFCUTemplate }));
			} else if (propResult.$val === beatsSmall084424VFCUController) {
				dom.html(terminalAssignmentView({ type: beatsSmall084424VFCUTemplate }));
			} else if (propResult.$val === compactVAVController) {
				dom.html(terminalAssignmentView({ type: compactVAVTemplate }));
			} else if (propResult.$val === irmLargeFCUController) {
				dom.html(terminalAssignmentView({ type: irmLargeFCUTemplate }));
			} else if (propResult.$val === irmRS4NRS5NFCUController) {
				dom.html(terminalAssignmentView({ type: irmRS4NRS5NFCUTemplate }));
			} else if (propResult.$val === irmRS3NFCUController) {
				dom.html(terminalAssignmentView({ type: irmRS3NFCUTemplate }));
			} else if (propResult.$val === beatsLarge1644230VFCUController) {
				dom.html(terminalAssignmentView({ type: beatsLarge1644230VFCUTemplate }));
			} else if (propResult.$val === beatsSmall0844230VFCUController) {
				dom.html(terminalAssignmentView({ type: beatsSmall0844230VFCUTemplate }));
			} else if (propResult.$val === beatsLarge168424VFCUController){
			    dom.html(terminalAssignmentView({ type: beatsLarge168424VFCUTemplate }));
			}
		}
		
		/**
		* Query the server side for the the terminal details supported by the current model.
		* @returns the result from ServerSide as a promise.
		*/
		function getIOTerminalDetails(comp) {
			var terminalServerSideclassName = "honIrmConfig:TerminalAssignmentServerSideCallHandler";
			var supportedFBNamesForTerminals = "getFunctionBlockNameSupportedForEachTerminal";

			return comp.serverSideCall({
				typeSpec: terminalServerSideclassName,
				methodName: supportedFBNamesForTerminals
			})
				.then(function(ioTerminalPropResult) {
					return Promise.resolve(ioTerminalPropResult);
				})
				.catch(function(err) {
					return Promise.reject(err);
				});
		}

		/**
		* Based on the ioTerminalPropResult, load all the dropdowns supported by the model
		* @returns a promise to confirm the completion of the operation.
		*/
		function loadIOTerminalDropdown(ioTerminalPropResult, dom) {
			terminals.length = 0;
			if (ioTerminalPropResult.getSlots().properties().is("baja:Component")) {
				ioTerminalPropResult.getSlots().dynamic().properties().each(function(terminalId) {
					var option = '';
					var terms = baja.SlotPath.unescape(terminalId.$slotName);
					var propName = terms.split("~");
					var term = propName[0];
					var initialSelection = propName[1];
					var termValue = terminalId.$val;
					terminals.push(term);
					var removeOption = '#' + term + ' option';
					$(removeOption).remove();
					option += '<option value=' + unassigned + '>' + unassigned + '</option>';
					if (termValue.getSlots().properties().is("baja:Component")) {
						termValue.getSlots().dynamic().properties().each(function(functionBlockList) {
							var dropDownAdd = baja.SlotPath.unescape(functionBlockList.$slotName);
							option += '<option value="' + dropDownAdd + '">' + dropDownAdd + '</option>';
						});
					} else {
						return Promise.reject(honIrmConfigLex.get('honIrmConfigLex.not.component'));
					}
					var ioTemrinal = getCorrespondingIOTerminal(dom, term);
					ioTemrinal.append(option);
					ioTemrinal.val(initialSelection);
				});
				var dropDownDivs = document.getElementsByClassName('terminal_container');
				var dropDownDivSize = window.getComputedStyle(dropDownDivs[0], null).getPropertyValue('font-size');
				var dropDownPxSizeInString = dropDownDivSize.split("px");
				dropDownReferencePxSize = parseInt(dropDownPxSizeInString);
				return Promise.resolve();
			}
			return Promise.reject(honIrmConfigLex.get('honIrmConfigLex.not.component'));
		}

		/**
		* Loop through all the terminals supported for the current model and add the subscription for 'Changed' & 'Click'
		* @returns a promise to confirm the completion of the operation.
		*/
		function subscribeToChangedCallBack(dom, widget) {
			for (var index in terminals) {
				if (index === "contains") {
					return;
				}
				var changedTerminal = getCorrespondingTerminalId(terminals[index]);
				var previousSelection;
				dom.on("click", changedTerminal, widget, function(event) {
					event.preventDefault();
					previousSelection = dom.find(this)[0].value;
					console.log("terminal before change " + previousSelection);
				});
				dom.on("change", changedTerminal, widget, function(event) {
					event.preventDefault();
					var currentSelection = dom.find(this)[0].value;
					console.log("terminal has been changed " + currentSelection);
					var isDuplicate = isDuplicateFound(dom, this, currentSelection, previousSelection);
					console.log("terminal has been changed " + isDuplicate);
					widget.setModified(true);
				});
			}
			return Promise.resolve();
		}

		/**
		* Loop through all the terminals supported for the current model and check if the 'previousSelection' is present
		* in any other terminal Dropdown. If present, then update 'previousSelection' in the other terminal.
		* else ask the user for confirmation to proceed with swap ot to revert the swapping. 
		* @returns True if Duplicate is found else false.
		*/
		function isDuplicateFound(dom, changedElement, currentSelection, previousSelection) {
			for (var index in terminals) {
				if (index === "contains") {
					return false;
				}
				var otherElement = terminals[index];
				if (changedElement.id === otherElement) {
					continue;
				}
				var termId = getCorrespondingTerminalId(otherElement);
				var existing = dom.find(termId)[0].value;
				if (existing === currentSelection && currentSelection !== unassigned) {
					// Check "previousSelection" is present in the drop down then set it as previousSelection otherwise set it as unassigned.
					if (isFunctionBlockExistsInDropDown(termId, previousSelection)) {
						console.log(previousSelection + " Option exists in " + otherElement.name);
						getCorrespondingIOTerminal(dom, otherElement).val(previousSelection);
					} else {
						console.log(previousSelection + " Option doesn't exists in " + otherElement.name);
						confirmAndRevertToOldValueBasedonUserConfirmation(dom, otherElement, changedElement, previousSelection);
					}
					return true;
				}
			}
			return false;
		}

		/**
		* Check if 'previousSelection' is present as an option in other terminal.
		* @returns true if previousSelection existing in the terminal else false.
		*/
		function isFunctionBlockExistsInDropDown(terminalId, previousSelection) {
			var exists = false;
			$(terminalId + ' option').each(function() {
				if (this.value === previousSelection) {
					exists = true;
				}
			});
			return exists;
		}
		
		/**
		* Update All the Terminal details in the component
		*/
		function updateDetailsOfTerminalInComponent() {
			for (var index in terminals) {
				if (index === "contains") {
					return;
				}
				var terminalPin = terminals[index];
				var terminalPinName = baja.SlotPath.escape(terminalPin);
				var functionBlockName = $(getCorrespondingTerminalId(terminalPin)).find(":selected").text();
				if (null !== changedItems.getSlot(terminalPinName)) {
					changedItems.remove(terminalPinName);
				}
				changedItems.add({
					slot: terminalPinName,
					value: functionBlockName,
				}).then(function(prop) {
					baja.outln('added a new Property named "' + prop + '"');
				})
					.catch(function(err) {
						baja.error('error adding Property: ' + err);
					});
			}
		}

		/**
		* Ask the user for confirmation on whether to proceed with marking 'otherElement' as Unassigned or update
		* 'changedElement' to the 'previousSelection'.
		*/
		function confirmAndRevertToOldValueBasedonUserConfirmation(dom, otherElement, changedElement, previousSelection) {
			dialogs.showYesNo({
				title: honIrmConfigLex.get('honIrmConfigLex.characteristic.not.supported.title'),
				content: honIrmConfigLex.get('honIrmConfigLex.characteristic.not.supported.content', previousSelection, otherElement),
				private: false
			})
				.yes(function() {
					console.log("Yes option clicked. Setting " + otherElement + " as unassigned");
					getCorrespondingIOTerminal(dom, otherElement).val(unassigned);
				}).no(function() {
					console.log("No option clicked. Reverting " + changedElement.id + " to " + previousSelection);
					dom.find(changedElement).val(previousSelection);
				});
		}


		/**
		* Save the component
		*/
		TerminalAssignment.prototype.doSave = function(comp) {
			var widget =this;
			updateDetailsOfTerminalInComponent();
			sendChangedDetailsToServerSide(comp, widget);
			changedItems = baja.$('baja:Component');
		};

		/**
		* Send the changes made by the user to the ServerSide as a BComponent.
		*/
		function sendChangedDetailsToServerSide(comp, widget) {
			var terminalServerSideclassName = "honIrmConfig:TerminalAssignmentServerSideCallHandler";
			var saveChangedDetails = "saveChangedTerminalAndRespond";

			return comp.serverSideCall({
				typeSpec: terminalServerSideclassName,
				methodName: saveChangedDetails,
				value: changedItems
			})
				.then(function(result) {
					console.log(result);
					handleResultFromServerSide(result, widget);
				})
				.catch(function(err) {
					return Promise.reject(err);
				});
		}

		/**
		* Based on the 'result'. Either show it to the user or not. If result is null don't show to user.'
		*/
		function handleResultFromServerSide(result, widget) {
			if (null !== result) {
				dialogs.showOk({
					title: honIrmConfigLex.get('honIrmConfigLex.error.saving.title'),
					content: result,
					private: false
				})
					.ok(function() {
						widget.setModified(true);
						console.log("OK option clicked.");
					});
			}
		}

		return TerminalAssignment;
	});
